﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Sockets;
using Hslu.Bluetooth;
using System.IO;
using System.Net.Sockets;
using RobotCtrl;
using Console = System.Console;

namespace BTServer
{
    static class Program
    {
        public static void Main()
        {
            Robot robot = setupRobot();
            TrackAdjustable parentTrack = new TrackAdjustable(1, 0);
            robot.Drive.RunATrack(parentTrack);

            BluetoothListener listener = setupBluetoothListener();

            if (listener != null)
            {
                Console.WriteLine("wait for incoming track data...");

                using (var file = new StreamWriter(new FileStream("commandLog.txt", FileMode.Create)))
                using (BluetoothClient client = listener.AcceptBluetoothClient())
                using (var reader = new StreamReader(client.GetStream()))
                {
                    while (true)
                    {
                        string line = reader.ReadLine();
                        if (line != null)
                        {
                            Console.WriteLine(line);

                            if (line == "exit")
                            {
                                break;
                            }
                            else
                            {
                                file.WriteLine(line);

                                Track track = TrackSerialization.Deserialize(line);
                                parentTrack.CurrentTrack = track;
                            }
                        }
                        System.Threading.Thread.Sleep(10);
                    }
                }
                listener.Stop();
            }
        }

        private static BluetoothListener setupBluetoothListener()
        {
            // check if Bluetooth-Stick is available
            if (!BluetoothRadio.IsSupported)
            {
                Console.WriteLine("No Bluetooth-Stick is available");
                Console.ReadLine();
                return null;
            }

            // Device visible for others
            // RadioMode setting not supported as long as the
            // WinCE image doesn't include the BthUtil.dll
            // BluetoothRadio.PrimaryRadio.Mode = 
            //     BluetoothRadioMode.Discoverable;

            BluetoothRadio.PrimaryRadio.Name = "Robot09";

            RobotID robotID = RobotIDs.Robot09;
            //Guid serviceId = new Guid("{FB4B43E4-0328-4056-82A5-7E03BE347082}");

            var listener = new BluetoothListener(robotID.Guid);
            listener.Start();
            Console.WriteLine("Service " + robotID.Guid.ToString() + " started.");

            return listener;
        }

        private static Robot setupRobot()
        {
            Robot robot = new Robot(RunMode.REAL);
            robot.Position.X = 0;
            robot.Position.Y = 0;
            robot.Drive.Power = true;
            //World.Robots.Add(robot);

            return robot;
        }

    }
}
