﻿using System;
using System.Collections.Generic;
using System.Text;

namespace InTheHand.Net.Bluetooth
{
    /// <summary>
    /// Specifies properties of a remote Bluetooth Device.
    /// </summary>
    /// -
    /// <remarks>
    /// <para>Originally from Win32 "bthdef.h" and used by struct
    /// BTH_DEVICE_INFO.flags. The flags are named BDIF_**.
    /// </para>
    /// </remarks>
    [Flags]
    public enum BluetoothDeviceInfoProperties // Really: uint
    {
        /// <summary>
        /// The address member contains valid data.
        /// </summary>
        Address = (0x00000001),
        /// <summary>
        /// The classOfDevice member contains valid data.
        /// </summary>
        Cod = (0x00000002),
        /// <summary>
        /// The name member contains valid data.
        /// </summary>
        Name = (0x00000004),
        /// <summary>
        /// The device is a remembered and authenticated device.
        /// The BDIF_PERSONAL flag is always set when this flag is set.
        /// </summary>
        Paired = (0x00000008),
        /// <summary>
        /// The device is a remembered device. If this flag is set and
        /// the BDIF_PAIRED flag is not set, the device is not authenticated.
        /// </summary>
        Personal = (0x00000010),
        /// <summary>
        /// The remote Bluetooth device is currently connected to the local radio.
        /// </summary>
        Connected = (0x00000020),

        //
        // support added in kb942567
        //
#pragma warning disable 1591 // One XmlDocs warning will do!
        ShortName = 0x00000040,
        Visible = 0x00000080,
        SspSupported = 0x00000100,
        SspPaired = 0x00000200,
        SspMitmProtected = 0x00000400,
        Rssi = 0x00001000,
        Eir = 0x00002000,
    }


    static class BDIFMasks
    {
        public const BluetoothDeviceInfoProperties AllOrig
            = BluetoothDeviceInfoProperties.Address | BluetoothDeviceInfoProperties.Cod
            | BluetoothDeviceInfoProperties.Name | BluetoothDeviceInfoProperties.Paired
            | BluetoothDeviceInfoProperties.Personal | BluetoothDeviceInfoProperties.Connected;
        public const BluetoothDeviceInfoProperties AllKb942567
            = AllOrig | BluetoothDeviceInfoProperties.ShortName
            | BluetoothDeviceInfoProperties.Visible | BluetoothDeviceInfoProperties.SspSupported
            | BluetoothDeviceInfoProperties.SspPaired | BluetoothDeviceInfoProperties.SspMitmProtected
            | BluetoothDeviceInfoProperties.Rssi | BluetoothDeviceInfoProperties.Eir;
    }
}