﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Windows.Forms;
using Chart;
using RobotCtrl;
using System.Drawing;
using RobotView;

namespace Test_Console_CE
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [MTAThread]
        static void Main()
        {
            //Application.Run(new ConsoleFormCE());
            //Application.Run(new MotorControl());
            //Application.Run(new ChartForm());
            //Application.Run(new DriveForm());
            //Application.Run(new TestWorld());

            //Testat 1:
            setupPylonRace();

            //Testat 2:
            //startController();
        }



        private static void setupPylonRace()
        {
            Robot robot = new Robot(RunMode.VIRTUAL);
            robot.Console.SwitchesChanged += new RobotCtrl.Console.SwitchesChangedEventHandler(Console_SwitchesChanged);
            robot.Position.X = 0;
            robot.Position.Y = 0;
            robot.Drive.Power = true;
            World.Robots.Add(robot);

            // Obstacles kreieren und in World einfügen
            World.Obstacles.Add(new ObstacleMap(Resource.Raceway,
                new RectangleF(-0.5f, -1.5f, 4.0f, 2.0f)));

            var worldForm = new WorldForm(World.Area);
            worldForm.Text = "..:: Pylon Race ::..";
            worldForm.Show();
            worldForm.Height = Screen.PrimaryScreen.WorkingArea.Height - 200;
            worldForm.Width = Screen.PrimaryScreen.WorkingArea.Height - 200;
            worldForm.Location = new Point(Screen.PrimaryScreen.WorkingArea.Width - worldForm.Width, 0);

            var consoleForm = new ConsoleForm(robot.Console);
            consoleForm.TopMost = true;
            consoleForm.Text = "Console";
            consoleForm.ControlBox = false;
            consoleForm.Show();
            consoleForm.Location = new Point(0, 0);

            Application.Run(worldForm);
            // Exit Main Thread (CF does not do this automatically)
            Application.Exit();
            //System.Threading.Thread.CurrentThread.Abort();

        }

        static void Console_SwitchesChanged(System.Collections.Specialized.BitVector32 oldValue, System.Collections.Specialized.BitVector32 newValue)
        {
            //SWITCH 0: Start race
            if (oldValue[1 << 0] != newValue[1 << 0])
            {
                if (newValue[1 << 0])
                {
                    foreach (Robot robot in World.Robots)
                    {
                        if (robot.Drive.CurrentTrack == null)
                        {
                            var trackPylons = new TrackPylons(.5, 1, robot.Radar, new TrackPylonsParams());
                            trackPylons.PylonDetectionChanged += trackPylons_PylonDetectionChanged;
                            robot.Drive.RunATrack(trackPylons);
                        }
                    }
                }
                else
                {
                    foreach (Robot robot in World.Robots)
                    {
                        robot.Drive.ResetPosition();
                    }
                }
            }

            //SWITCH 4: Define nominal speed
            if (newValue[1 << 3] != oldValue[1 << 3])
            {
                if (newValue[1 << 3])
                {
                    foreach (Robot robot in World.Robots)
                    {
                        if(robot.Drive.CurrentTrack != null)
                            robot.Drive.CurrentTrack.nominalSpeed = 0.9;
                    }
                }
                else
                {
                    foreach (Robot robot in World.Robots)
                    {
                        if (robot.Drive.CurrentTrack != null)
                            robot.Drive.CurrentTrack.nominalSpeed = 0.5;
                    }
                }
            }
        }

        static void trackPylons_PylonDetectionChanged(object sender, TrackPylons.PylonDetectionArgs e)
        {
            World.Robots[0].Console.OverrideLampInvocation(0, e.Detected); 
        }
    }
}