using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;

namespace ServerPattern
{
    public abstract class AbstractServer
    {
        private Boolean running = true;
        private Socket socket;

        public int Port { get; private set; }
        public string Host { get; private set; }
        public const int DEFAULTPORT = 4711;
        public const string DEFAULTHOST = "localhost";


        protected void Run()
        {
            var executor = CreateExecutor();
            socket = CreateServerSocket();
            while (running)
            {
                AbstractHandler handler = CreateHandler(socket.Accept());
                executor(handler.Run);
            }
        }

        virtual protected Action<Action> CreateExecutor()
        {
            return action => ThreadPool.QueueUserWorkItem(ignore => action());
        }

        protected Socket CreateServerSocket()
        {
            IPAddress ipAddress = Dns.GetHostEntry(this.Host).AddressList[0];
            TcpListener listener = new TcpListener(ipAddress, this.Port);
            listener.Start();
            return listener.Server;
        }

        public void Start()
        {
            this.Start(DEFAULTPORT, null);
        }

        public void Start(int port)
        {
            this.Start(port, null);
        }

        public void Start(int port, string host)
        {
            if (port <= 0 || port >= (1 << 16) - 1)
                throw new ArgumentOutOfRangeException("port");

            this.Port = port;
            this.Host = host ?? DEFAULTHOST;

            Thread server = new Thread(Run);
            server.Start();
        }

        abstract protected AbstractHandler CreateHandler(Socket client);
    }
}
